#include "NVSExplorer.hpp"

// C++ system level
#include <set>
#include <cstring>     // memset, memcpy, strncmp
// #include <cstdlib>
#include <cinttypes>
// #include <functional>

// ESP32 specific
#include "esp_log.h"
#include "nvs_flash.h"
// #include "esp_err.h"
// #include "driver/uart.h"

// project specific
// #include ""

// qthing stuff
// #include <qthing>





namespace SiliconTorch {

  namespace NVSExplorer {


    const char* TAG = "NVSExplorer";


    NVSExplorer::NVSExplorer() {

      ESP_LOGI(TAG, "Starting NVSExplorer[📡]");

      nvs_flash_init();
    }


    void NVSExplorer::BullShit() {


      ESP_LOGW(TAG, "Scanning 👀");

      nvs_iterator_t it = nvs_entry_find("nvs", NULL, NVS_TYPE_ANY);

      while (it != NULL) {
        nvs_entry_info_t info;
        nvs_entry_info(it, &info);
        it = nvs_entry_next(it);

        ESP_LOGW(TAG, "namespace[ %s ]   key[ %s ]   type[ %d ]", info.namespace_name, info.key, info.type);
      }


    }


    std::set<std::string> NVSExplorer::listKeys(std::string& nameSpace) {

      std::set<std::string> keys;


      nvs_iterator_t it = nvs_entry_find("nvs", nameSpace.c_str(), NVS_TYPE_ANY);

      while (it != NULL) {
        nvs_entry_info_t info;
        nvs_entry_info(it, &info);
        it = nvs_entry_next(it);

        ESP_LOGW(TAG, "namespace[ %s ]   key[ %s ]   type[ %d ]", info.namespace_name, info.key, info.type);  // TODO: remove after dev!

        keys.insert(info.key);
      }

      return keys;
    }

    std::set<std::string> NVSExplorer::listNamespaces() {

      std::set<std::string> namespaces;


      nvs_iterator_t it = nvs_entry_find("nvs", NULL, NVS_TYPE_ANY);

      while (it != NULL) {
        nvs_entry_info_t info;
        nvs_entry_info(it, &info);
        it = nvs_entry_next(it);

        ESP_LOGW(TAG, "namespace[ %s ]   key[ %s ]   type[ %d ]", info.namespace_name, info.key, info.type);  // TODO: remove after dev!

        namespaces.insert(info.namespace_name);
      }

      return namespaces;
    }



    std::string toString(std::set<std::string>& set, std::string delimiter) {

      uint32_t length = 0;

      // caching this temporarily
      // TODO: do we need this or would the compiler be smart enough to cache it itself?
      // or couldn't he cache it itself because C++ is strict and it is a method-call?
      // questions over questions………
      uint32_t delimiterLength = delimiter.length();

      for (auto const& item : set) length += item.length() + delimiterLength;  // TODO: replace delimiterLength -> delimiter.length()  ?


      length++;
      char str[length];
      char* ptr = str;
      memset(str, 0x00, length);

      for (auto const& item : set) {
        
        uint32_t len = item.length();

        std::memcpy(ptr, item.c_str(), len);
        ptr += len;

        std::memcpy(ptr, delimiter.c_str(), delimiterLength);
        ptr += delimiterLength;
      }

      ptr -= delimiterLength;
      memset(ptr, 0x00, delimiterLength);  // clear out the trailing delimiter

      return std::string(str);
    }


  }
}
